"""Various ferda utils."""
import asyncio
from typing import Any, Awaitable


class AbcAttributeType:
    """
    Abstract attribute type.

    Use instance of this class as default value of ABC
    attribute in order to enforce class attribute definition.
    """

    __isabstractmethod__ = True


# Abstract attribute singleton
AbcAttribute: Any = AbcAttributeType()


def removeprefix(value: str, prefix: str) -> str:
    """Remove string prefix if it exists."""
    # XXX: Replace with `str.removeprefix` in Python 3.9+
    if value.startswith(prefix):
        return value.replace(prefix, "", 1)
    else:
        return value


async def await_with_semaphore(call: Awaitable, semaphore: asyncio.Semaphore) -> Any:
    """Await awaitable with semaphore."""
    async with semaphore:
        return await call
